<?php

namespace Drupal\set_field_defaults\Form;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Config form for setting default field values.
 */
class SetDefaultsConfigForm extends ConfigFormBase {

  /**
   * Field types and their labels.
   */
  private array $types = [
    'string' => 'Text (plain)',
    'string_long' => 'Text (plain, long)',
    'text' => 'Text (formatted)',
    'text_long' => 'Text (formatted, long)',
    'text_with_summary' => 'Text (formatted, long, with summary)',
    'decimal' => 'Number (decimal)',
    'float' => 'Number (float)',
    'integer' => 'Number (integer)',
    'link' => 'Link',
    'email' => 'Email',
  ];

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['set_field_defaults.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'set_field_defaults_config_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('set_field_defaults.settings');

    $form['set_default_on_create'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('<b>Automatically Set Default Values</b>'),
      '#description' => $this->t('Enabling this ensures that newly created fields are assigned default values automatically.<br><b>Note:</b> This will also preselect the "Set default value" option in the field configuration page.'),
      '#default_value' => $config->get('set_default_on_create') ?? 0,
    ];

    $form['table'] = [
      '#type' => 'table',
      '#header' => [$this->t('Field'), $this->t('Default Value')],
      '#empty' => $this->t('No field types available.'),
    ];

    foreach ($this->types as $type => $label) {
      $form['table'][$type] = [
        'label' => [
          '#markup' => $label,
        ],
        $this->getFieldElement($type, $config),
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * Helper function to get the field element based on type.
   */
  protected function getFieldElement(string $type, $config) {
    $number_steps = [
      'decimal' => 'any',
      'float' => 'any',
      'integer' => 1,
    ];

    switch ($type) {
      case 'string':
      case 'text':
        return [
          '#type' => 'textfield',
          '#default_value' => $config->get($type) ?? '',
          '#maxlength' => 255,
        ];

      case 'string_long':
      case 'text_long':
      case 'text_with_summary':
        return [
          '#type' => 'textarea',
          '#default_value' => $config->get($type) ?? '',
          '#rows' => 2,
        ];

      case 'decimal':
      case 'float':
      case 'integer':
        return [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['number-fields-container'],
            'style' => ['display: inline-flex; gap: 2rem;'],
          ],
          'min' => [
            '#type' => 'number',
            '#title' => $this->t('Minimum'),
            '#default_value' => $config->get($type . '_min') ?? '',
            '#step' => $number_steps[$type],
          ],
          'default_value' => [
            '#type' => 'number',
            '#title' => $this->t('Default value'),
            '#default_value' => $config->get($type . '_default_value') ?? '',
            '#step' => $number_steps[$type],
          ],
          'max' => [
            '#type' => 'number',
            '#title' => $this->t('Maximum'),
            '#default_value' => $config->get($type . '_max') ?? '',
            '#step' => $number_steps[$type],
          ],
        ];

      case 'link':
        return [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['link-fields-container'],
            'style' => ['display: inline-flex; gap: 2rem;'],
          ],
          'uri' => [
            '#type' => 'textfield',
            '#title' => $this->t('URL'),
            '#default_value' => $config->get($type . '_url') ?? '',
            '#description' => $this->t('Enter an external URL (e.g., https://example.com) or use %front, %nolink, or # for internal paths.', [
              '%front' => '<front>',
              '%nolink' => '<nolink>',
            ]),
          ],
          'link_text' => [
            '#type' => 'textfield',
            '#title' => $this->t('Link text'),
            '#default_value' => $config->get($type . '_link_text') ?? '',
          ],
        ];

      case 'email':
        return [
          '#type' => 'email',
          '#default_value' => $config->get($type) ?? '',
        ];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    foreach ($this->types as $type => $label) {
      $hierarchy = $form_state->getValue('table')[$type][0];

      switch ($type) {
        case 'integer':
        case 'decimal':
        case 'float':
          $min = $hierarchy['min'];
          $max = $hierarchy['max'];
          $default_value = $hierarchy['default_value'];

          $fields = ['min' => 'Minimum', 'max' => 'Maximum', 'default_value' => 'Default value'];

          foreach ($fields as $key => $label) {
            if (!empty($hierarchy[$key]) && !is_numeric($hierarchy[$key])) {
              $form_state->setErrorByName($type, $this->t('Please enter a numeric value for @type: @label.', [
                '@type' => $type,
                '@label' => $label,
              ]));
            }
          }

          if ($min !== '' && $min !== NULL && !is_numeric($min)) {
            $form_state->setErrorByName($type, $this->t('Please enter a numeric value for @type: Minimum.', ['@type' => $type]));
          }

          if ($max !== '' && $max !== NULL && !is_numeric($max)) {
            $form_state->setErrorByName($type, $this->t('Please enter a numeric value for @type: Maximum.', ['@type' => $type]));
          }

          if ($default_value !== '' && $default_value !== NULL && !is_numeric($default_value)) {
            $form_state->setErrorByName($type, $this->t('Please enter a numeric value for @type: Default value.', ['@type' => $type]));
          }

          if (isset($min, $max, $default_value) && is_numeric($min) && is_numeric($max) && is_numeric($default_value)) {
            if ($max < $min) {
              $form_state->setErrorByName($type, $this->t('Maximum value must be greater than or equal to Minimum value for @type.', ['@type' => $type]));
            }

            if ($default_value < $min || $default_value > $max) {
              $form_state->setErrorByName($type, $this->t('Default value must be between Minimum and Maximum for @type.', ['@type' => $type]));
            }
          }
          break;

        case 'link':
          $allowed_internal_values = ['#', '<front>', '<nolink>'];
          $uri_value = trim($hierarchy['uri'] ?? '');

          if (!empty($uri_value) && !in_array($uri_value, $allowed_internal_values) && !UrlHelper::isValid($uri_value, TRUE)) {
            $form_state->setErrorByName($type, $this->t('@label must be a valid URL or one of the following: #, <front>, <nolink>.', [
              '@label' => $label,
            ]));
          }
          break;

        case 'email':
          if (!empty($hierarchy) && !filter_var($hierarchy, FILTER_VALIDATE_EMAIL)) {
            $form_state->setErrorByName($type, $this->t('@label must be a valid email address.', ['@label' => $label]));
          }
          break;
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $config = $this->config('set_field_defaults.settings');

    foreach ($this->types as $type => $label) {
      $hierarchy = $form_state->getValue('table')[$type][0];
      if (in_array($type, ['decimal', 'float', 'integer'])) {
        $config->set($type . '_min', $hierarchy['min']);
        $config->set($type . '_max', $hierarchy['max']);
        $config->set($type . '_default_value', $hierarchy['default_value']);
      }
      elseif ($type == 'link') {
        $config->set($type . '_url', $hierarchy['uri']);
        $config->set($type . '_link_text', $hierarchy['link_text']);
      }
      else {
        $config->set($type, $hierarchy);
      }
    }

    $config->set('set_default_on_create', $form_state->getValue('set_default_on_create'));

    $config->save();
  }

}
